package com.persagy.dmp.admin.config.shiro;

import com.persagy.common.web.ServletUtils;
import com.persagy.dmp.admin.entity.AdminUser;
import com.persagy.dmp.admin.entity.QAdminUser;
import com.persagy.dmp.admin.repository.AdminUserRepository;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.apache.shiro.web.filter.AccessControlFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Optional;

@Component("authc")
public class DmpAccessControlFilter extends AccessControlFilter {

	public final Logger logger = LoggerFactory.getLogger(DmpAccessControlFilter.class);

	private final static String HEADER_TOKEN = "dmp-token";

	@Autowired
	private StringRedisTemplate redis;

	@Autowired
	private AdminUserRepository userRepository;

	//判断是否拦截，false为拦截，true为允许继续访问
	@Override
	protected boolean isAccessAllowed(ServletRequest req, ServletResponse resp, Object mappedValue) throws Exception {
		HttpServletRequest request = (HttpServletRequest) req;
		HttpServletResponse response = (HttpServletResponse) resp;
		logger.info("===================================== isAccessAllowed");
		String uri = request.getRequestURI();
		logger.info("开始请求地址拦截 {}", uri);
//		ServletUtils.showHeaders(request);
//		ServletUtils.showCookies(request);

		if (uri.equals(this.getLoginUrl())) {
			// 登录地址,直接放行
			return true;
		}

		String token = request.getHeader(HEADER_TOKEN);
		if (token == null) {
			ServletUtils.writeJson(response, "{\"result\":\"no_token\",\"message\":\"no token\"}");
			return false;
		}

		// token不存在
		String key = ShiroContants.KEY_TOKEN_USER + token;
		if (!redis.hasKey(key)) {
			ServletUtils.writeJson(response, "{\"result\":\"unauthc\",\"message\":\"token invalid\"}");
			return false;
		}

		try {
			Subject subject = SecurityUtils.getSubject();
			if (subject.isAuthenticated()) {
				Object principal = subject.getPrincipal();
				if (principal instanceof UserModel) {
					// 防止前端切换用户, 用token做密码
					UserModel userModel = (UserModel) principal;
					if (!userModel.getToken().equalsIgnoreCase(token)) {
						return checkToken(subject, token, response);
					}
				} else {
					return checkToken(subject, token, response);
				}
			} else {
				return checkToken(subject, token, response);
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("login check error: {}", e);
			return false;
		}
		return true;
	}

	private boolean checkToken(Subject subject, String token, HttpServletResponse response) {

		String key = ShiroContants.KEY_TOKEN_USER + token;
		String userIdStr = redis.opsForValue().get(key);
		Integer userId = Integer.valueOf(userIdStr);


		Optional<AdminUser> one = userRepository.findOne(QAdminUser.adminUser.id.eq(userId));
		if (!one.isPresent()) {
			// 用户不存在
			ServletUtils.writeJson(response, "{\"result\": \"no_user\"}");
			return false;
		}
		AdminUser user = one.get();
		if (user.getStatus() != 1) {
			// 用户被禁用
			ServletUtils.writeJson(response, "{\"result\": \"invalid_user\"}");
			return false;
		} else {
			// 更新用户名
			String name = user.getRealname() == null ? user.getUsername() : user.getRealname();

			// 用ssoId作为用户名
			UsernamePasswordToken shiroToken = new UsernamePasswordToken(user.getUsername(), token);
			subject.login(shiroToken);
		}
		return true;
	}

	//上面的方法返回false后(被拦截)，会进入这个方法；这个方法返回false表示处理完毕(不放行)；返回true表示需要继续处理(放行)
	// 身份认证逻辑已在isAccessAllowed处理,这里直接返回false
	@Override
	protected boolean onAccessDenied(ServletRequest request, ServletResponse response) throws Exception {
		HttpServletRequest req = (HttpServletRequest) request;
		logger.info("===================================== onAccessDenied {}", req.getRequestURI());
		return false;
	}
}
